@@ -427,118 +427,6 @@ bool Bitmap::find_rows_and_columns() throw()
   }
 
 
-int Bitmap::horizontalify( bool verbose ) throw()
-  {
-  const int slope_max = _width / 25;
-  int slope_best = 0;			// not a true slope
-  long mean = 0;
-  long long variance_best = 0;		// not a true variance
-
-  if( slope_max == 0 ) return 0;
-
-  for( int row = 0; row < _height; ++row )
-    for( int col = 0; col < _width; ++col )
-      if( data[row][col] ) ++mean;
-  mean /= _height;
-
-  for( int row = 0; row < _height; ++row )
-    {
-    long c = -mean;
-    for( int col = 0; col < _width; ++col )
-      if( data[row][col] ) ++c;
-    variance_best += ( c * c );
-    }
-
-if( verbose )
-  {
-  std::fprintf( stderr, "mean = %ld\n", mean );
-  std::fprintf( stderr, "slope   0   variance %Ld\n", variance_best );
-  }
-
-  bool try_inc = true, try_dec = true;
-  for( int slope = 1; slope <= slope_max; ++slope )
-    {
-    long long variance = 0;
-    int step = _width / slope; if( step * slope < _width ) ++step;
-
-    if( try_inc )
-      {
-      for( int row = 0; row < _height - slope; ++row )
-        {
-        long c = -mean; int y = 0;
-        for( int col = 0; col < _width; ++col )
-          {
-          if( data[row + y][col] ) ++c;
-          if( col % step == step - 1 ) ++y;
-          }
-        variance += ( c * c );
-        }
-      if( variance > variance_best )
-        { variance_best = variance; slope_best = slope; }
-      else if( variance < variance_best - (variance_best / 10) )
-        try_inc = false;
-      if( verbose )
-        std::fprintf( stderr, "slope %3d   variance %Ld\n", slope, variance );
-      }
-
-    if( try_dec )
-      {
-      variance = 0;
-      for( int row = slope; row < _height; ++row )
-        {
-        long c = -mean; int y = 0;
-        for( int col = 0; col < _width; ++col )
-          {
-          if( data[row - y][col] ) ++c;
-          if( col % step == step - 1 ) ++y;
-          }
-        variance += ( c * c );
-        }
-      if( variance > variance_best )
-        { variance_best = variance; slope_best = -slope; }
-      else if( variance < variance_best - (variance_best / 10) )
-        try_dec = false;
-      if( verbose )
-        std::fprintf( stderr, "slope %3d   variance %Ld\n", -slope, variance );
-      }
-    }
-
-  if( slope_best > 0 )
-    {
-    int row = 0;
-    int step = _width / slope_best;
-    if( step * slope_best < _width ) ++step;
-    for( ; row < _height - slope_best; ++row )
-      {
-      for( int col = 0, y = 0; col < _width; ++col )
-        {
-        data[row][col] = data[row + y][col];
-        if( col % step == step - 1 ) ++y;
-        }
-      }
-    for( ; row < _height; ++row )
-      for( int col = 0; col < _width; ++col ) data[row][col] = false;
-    }
-  else if( slope_best < 0 )
-    {
-    int row = _height - 1;
-    int step = _width / -slope_best;
-    if( step * -slope_best < _width ) ++step;
-    for( ; row >= -slope_best; --row )
-      {
-      for( int col = 0, y = 0; col < _width; ++col )
-        {
-        data[row][col] = data[row - y][col];
-        if( col % step == step - 1 ) ++y;
-        }
-      }
-    for( ; row >= 0; --row )
-      for( int col = 0; col < _width; ++col ) data[row][col] = false;
-    }
-  return slope_best;
-  }
-
-
 void Bitmap::save( FILE * f, char filetype ) const throw()
   {
   if( filetype != '1' && filetype != '4' ) filetype = '4';
