@@ -24,7 +24,7 @@ class Blockmap
   int _height;
   int _width;
   std::vector< std::vector< int > > data;
-  std::vector< Block > _block_vector;
+  std::vector< std::vector< Block > > _block_matrix;
 
   int generate_black_id();
   int generate_white_id();
@@ -42,9 +42,9 @@ public:
   int height() const throw() { return _height; }
   int width()  const throw() { return _width;  }
   int id( int row, int col ) const throw() { return data[row][col]; }
-  const std::vector< Block > & block_vector() const throw()
-    { return _block_vector; }
-  int blocks() const throw() { return _block_vector.size(); }
+  const std::vector< std::vector< Block > > & block_matrix() const throw()
+    { return _block_matrix; }
+  int blocks() const throw();
 
   void print( FILE * outfile, int debug_level ) const throw();
   };
