@@ -41,10 +41,17 @@ void Character::recognize12( const Charset & charset, int charbox_vcenter ) thro
 
   if( Ocrad::similar( a1, a2, 10 ) )
     {
-    if( width() > height() || Ocrad::similar( width(), height(), 50 ) )
-      { add_guess( '=', 0 ); return; }
-    if( b1.height() >= 2 * b1.width() ) { add_guess( '|', 0 ); return; }
-    add_guess( ':', 0 ); return;
+    if( !b1.blocks() && !b2.blocks() &&
+        2 * a1 > b1.size() && 2 * a2 > b2.size() )
+      {
+      if( width() > height() || Ocrad::similar( width(), height(), 40 ) )
+        { add_guess( '=', 0 ); return; }
+      if( Ocrad::similar( b1.width(), b1.height(), 20, 2 ) &&
+          Ocrad::similar( b2.width(), b2.height(), 20, 2 ) )
+        add_guess( ':', 0 );
+      return;
+      }
+    return;
     }
   if( Ocrad::similar( a1, a2, 60 ) )
     {
@@ -122,7 +129,8 @@ void Character::recognize12( const Charset & charset, int charbox_vcenter ) thro
           }
         }
 
-      if( code == '.' && f2.bp.minima( b2.height() / 4 ) == 1 )
+      if( code == '.' && f2.bp.minima( b2.height() / 4 ) == 1 &&
+          b1.bottom() <= b2.top() )
         {
         int hdiff;
         if( b2.bottom_hook( &hdiff ) && std::abs( hdiff ) >= b2.height() / 2 )
@@ -186,9 +194,12 @@ void Character::recognize12( const Charset & charset, int charbox_vcenter ) thro
       }
     return;
     }
+
+  if( b1.bottom() <= b2.top() )
+  {
   int code = f2.test_solid( charbox_vcenter );
-  if( code == '.' || 
-      ( code && Ocrad::similar( b2.height(), b2.width(), 50 ) ) )
+  if( !b1.blocks() && ( code == '.' || 
+        ( code && Ocrad::similar( b2.height(), b2.width(), 50 ) ) ) )
     {
     if( Ocrad::similar( b1.width(), b2.width(), 50 ) )
       { add_guess( '!', 0 ); return; }
@@ -207,3 +218,4 @@ void Character::recognize12( const Charset & charset, int charbox_vcenter ) thro
         }
       }
   }
+  }
