@@ -203,6 +203,8 @@ int Features::test_235Esz( const Charset & charset ) const throw()
   static const int ucoff[csize] = { 0, +1, -1, -1, +1 };
   static const int lcoff[csize] = { 0, +1, -1, +1, -1 };
   const Block & b = *_block;
+  if( b.width() < 9 || b.height() > 3 * b.width() ) return 0;
+
   const Blockmap & bm = *_blockmap;
   int lrow1 = 0, urow2 = 0, lrow2 = 0, urow3 = 0;
   int lcol1 = 0, ucol2 = 0, lcol2 = 0, ucol3 = 0, black_section = 0;
@@ -415,13 +417,13 @@ int Features::test_4ADQao( const Charset & charset, int charbox_vcenter ) const
       if( c == 4 && Ocrad::similar( left_delta, right_delta, 40 ) &&
           lp.ispit() ) return '@';
       }
-    if( tp.minima() == 1 && bp.minima() == 2 && !lp.isconvex() ) return 'A';
+    if( tp.minima() == 1 && bp.istip() && !lp.isconvex() ) return 'A';
     }
   if( Ocrad::similar( left_delta, right_delta, 40 ) )
     {
     if( !bp.isconvex() && rp.ispit() && bp[1] < 2 ) return 'D';
     if( bp.minima() > 1 || rp.minima() > 1 ) return 'Q';
-    if( b.bottom() < charbox_vcenter ) return UCS::DEG;
+    if( b.vpos( 80 ) < charbox_vcenter ) return UCS::DEG;
     return 'o';
     }
   if( left_delta > right_delta && rp.ispit() && tp.minima() == 1 ) return 'D';
@@ -599,17 +601,21 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
         Ocrad::similar( vbar(0).left() - b.left(),
                         b.right() - vbar(0).right(), 30, 2 ) )
       {
-      if( rp.istip() && ( lp.istip() || lp.isflats() ) )
+      if( rp.istip() && lp.istip() )
         {
-        if( lp[lp.pos(40)] > lp[lp.pos(60)] + 1 &&
+        if( b.height() <= 3 * b.width() &&
+            lp[lp.pos(40)] > lp[lp.pos(60)] + 1 &&
             rp[rp.pos(60)] > rp[rp.pos(40)] + 1 ) return 'z';
         return 'I';
         }
       if( rp.isflats() && ( lp.istip() || lp.isflats() ) ) return 'l';
-      if( rp.istip() && lp.ispit() && Ocrad::similar( lp.iminimum(), rp.pos(50), 10 ) )
-        { if( lp.isvpit() ) return '{'; else return '('; }
-      if( lp.istip() && rp.ispit() && Ocrad::similar( rp.iminimum(), rp.pos(50), 10 ) )
-        { if( rp.isvpit() ) return '}'; else return ')'; }
+      if( b.height() > 3 * wp.max() )
+        {
+        if( rp.istip() && lp.ispit() && Ocrad::similar( lp.iminimum(), rp.pos(50), 10 ) )
+          { if( lp.isvpit() ) return '{'; else return '('; }
+        if( lp.istip() && rp.ispit() && Ocrad::similar( rp.iminimum(), rp.pos(50), 10 ) )
+          { if( rp.isvpit() ) return '}'; else return ')'; }
+        }
       if( 2 * b.height() > 3 * b.width() )
         if( rp.isflats() || rp.minima() == 1 )
           if( vbar(0).right() >= b.hpos( 70 ) ||
@@ -621,12 +627,17 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
     if( vbar(0).left() <= b.left() + 1 && b.height() > 2 * b.width() &&
         rp.istip() )
       {
-      int urow = b.seek_top( b.vcenter(), b.hcenter() );
-      int lrow = b.seek_bottom( b.vcenter(), b.hcenter() );
-      int ucol = b.seek_right( urow, b.hcenter() );
-      int lcol = b.seek_right( lrow, b.hcenter() );
-      if( ucol < b.right() && lcol < b.right() )
-        return 'C';
+      if( 2 * rp[rp.pos(50)] > b.width() )
+        {
+        int row = b.seek_top( b.vcenter(), b.hcenter() );
+        int col = b.seek_right( row, b.hcenter() );
+        if( col < b.right() )
+          {
+          row = b.seek_bottom( b.vcenter(), b.hcenter() );
+          col = b.seek_right( row, b.hcenter() );
+          if( col < b.right() ) return 'C';
+          }
+        }
       return '[';
       }
     if( vbar(0).right() >= b.right() - 1 )
@@ -854,7 +865,10 @@ int Features::test_HKMNUuvwYy() const throw()
             10 * hbar(0).h_distance( b.hcenter() ) <= b.width() )
           return 'H';
         if( vscan()[rp.pos(60)] == 4 || vscan()[rp.pos(70)] == 4 ) return 'w';
-        if( b.width() > 2 * b.height() ) return '~'; else return 'N';
+        if( 2 * b.width() > 5 * b.height() ) return '~';
+        if( vbars() <= 2 || ( vbars() == 3 && b.height() >= b.width() ) )
+          return 'N';
+        return 0;
         }
       if( 3 * lg < 2 * rg && lg < b.width() / 4 && rg > b.width() / 4 &&
           rp.isctip() && tp.minima( b.height() / 8 ) == 2 ) return 'K';
@@ -1071,8 +1085,9 @@ int Features::test_misc() const throw()
       if( hbars() == 2 && hbar(0).bottom() < b.vpos( 25 ) &&
           hbar(1).bottom() >= b.bottom() - 1 )
         {
+        if( hbar(0).right() <= hbar(1).hcenter() ) return 0;
         if( 3 * hbar(0).width() <= 2 * hbar(1).width() ) return 'l';
-        else return 'I';
+        return 'I';
         }
       }
     if( hbars() == 1 ||
@@ -1092,12 +1107,13 @@ int Features::test_misc() const throw()
             3 * hbar(0).width() > 4 * hbar(2).width() ) ) ) return 'F';
     }
 
-  if( b.height() >= 3 * wp.max() )
+  if( b.height() > 3 * wp.max() )
     {
     if( rp.istip() && lp.ispit() )
       { if( lp.isvpit() ) return '{'; else return '('; }
     if( lp.istip() && rp.ispit() )
       { if( rp.isvpit() ) return '}'; else return ')'; }
+    if( b.width() > 2 * wp.max() && rp.isconvex() ) return ')';
     }
 
   return 0;
