@@ -19,7 +19,7 @@
     Return values: 0 for a normal exit, 1 for environmental problems
     (file not found, invalid flags, I/O errors, etc), 2 to indicate a
     corrupt or invalid input file, 3 for an internal consistency error
-    (eg, bug) which caused Ocrad to panic.
+    (eg, bug) which caused ocrad to panic.
 */
 
 #include <cctype>
@@ -30,6 +30,8 @@
 #include <getopt.h>
 #include "common.h"
 #include "rectangle.h"
+#include "vrhomboid.h"
+#include "track.h"
 #include "bitmap.h"
 #include "block.h"
 #include "blockmap.h"
@@ -40,9 +42,9 @@
 
 namespace {
 
-// Date of this version: 2004-02-09
+// Date of this version: 2004-05-23
 
-const char *const program_version = "0.7";
+const char *const program_version = "0.8";
 const char *const program_year    = "2004";
 
 
@@ -86,7 +88,6 @@ void show_help( const char * program_name, bool verbose ) throw()
     std::printf( "  -1, -4                   pbm output file type (debug)\n");
     std::printf( "  -C, --copy               'copy' input to output (debug)\n");
     std::printf( "  -D, --debug=<level>      (0-100) output intermediate data (debug)\n");
-//    std::printf( "  -L, --level              level input image before doing OCR\n");
     std::printf( "  -R <scale>               reduce input image by <scale> (debug)\n");
     std::printf( "  -S <type>                make a 'special file' (debug)\n");
     std::printf( "  -T <threshold>           set threshold for -R option (debug)\n");
@@ -102,19 +103,30 @@ void ocr( const Bitmap & page_image, const Charset & charset,
   if( !blockmap.blocks() ) return;
 
   if( control.debug_level >= 90 )
-    { blockmap.print( control.outfile, control.debug_level ); return; }
+    {
+    if( control.outfile )
+      blockmap.print( control.outfile, control.debug_level );
+    return;
+    }
 
-  Textblock textblock( charset, blockmap.block_vector() );
+  Textblock textblock( charset, blockmap.block_matrix() );
 
   if( control.debug_level >= 86 )
     {
-    bool graph = ( control.debug_level >= 88 );
-    bool recursive = ( control.debug_level & 1 );
-    textblock.dprint( control, graph, recursive );
+    if( control.outfile )
+      {
+      bool graph = ( control.debug_level >= 88 );
+      bool recursive = ( control.debug_level & 1 );
+      textblock.dprint( control, graph, recursive );
+      }
+    return;
     }
 
-  if( control.debug_level == 0 ) textblock.print( control );
-  if( control.exportfile ) textblock.xprint( control );
+  if( control.debug_level == 0 )
+    {
+    if( control.outfile ) textblock.print( control );
+    if( control.exportfile ) textblock.xprint( control );
+    }
   }
 
 
@@ -136,35 +148,42 @@ int process_file( FILE *infile, const Charset & charset,
       return 1;
       }
 
-    if( control.level )
-      std::fprintf( stderr, "\nslope_best = %d\n",
-                    page_image.horizontalify( control.verbose ) );
-
     if( control.scale != 0 )
       {
-      Bitmap reduced( page_image, control.scale, control.threshold );
-      reduced.save( control.outfile, control.filetype ); return 0;
+      if( control.outfile )
+        {
+        Bitmap reduced( page_image, control.scale, control.threshold );
+        reduced.save( control.outfile, control.filetype );
+        }
+      return 0;
       }
     else if( control.specialtype != 0 )
       {
-      Bitmap::type t;
-      if( control.specialtype == 'v' ) t = Bitmap::vertical_histogram ;
-      else if( control.specialtype == 'h' ) t = Bitmap::horizontal_histogram ;
-      else if( control.specialtype == 'g' ) t = Bitmap::connected_ground ;
-      else { show_error( 0, "bad special type" ); return 1; }
-      Bitmap sb( page_image, t );
-      sb.save( control.outfile, control.filetype ); return 0;
+      if( control.outfile )
+        {
+        Bitmap::type t;
+        if( control.specialtype == 'v' ) t = Bitmap::vertical_histogram ;
+        else if( control.specialtype == 'h' ) t = Bitmap::horizontal_histogram ;
+        else if( control.specialtype == 'g' ) t = Bitmap::connected_ground ;
+        else { show_error( 0, "bad special type" ); return 1; }
+        Bitmap sb( page_image, t );
+        sb.save( control.outfile, control.filetype );
+        }
+      return 0;
       }
     else if( control.copy )
       {
-      if( control.rindex < 0 && page_image.rectangles() == 1 )
-        { page_image.save( control.outfile, control.filetype ); return 0; }
-      for( int c = 0; c < page_image.rectangles(); ++c )
-        if( control.rindex < 0 || control.rindex == c )
-          {
-          Bitmap bitmap( page_image, page_image.rectangle_vector()[c] );
-          bitmap.save( control.outfile, control.filetype );
-          }
+      if( control.outfile )
+        {
+        if( control.rindex < 0 && page_image.rectangles() == 1 )
+          { page_image.save( control.outfile, control.filetype ); return 0; }
+        for( int c = 0; c < page_image.rectangles(); ++c )
+          if( control.rindex < 0 || control.rindex == c )
+            {
+            Bitmap bitmap( page_image, page_image.rectangle_vector()[c] );
+            bitmap.save( control.outfile, control.filetype );
+            }
+        }
       return 0;
       }
 
@@ -227,13 +246,12 @@ int main( int argc, char *argv[] ) throw()
       {"help", no_argument, 0, 'h'},
       {"invert", no_argument, 0, 'i'},
       {"layout", required_argument, 0, 'l'},
-      {"level", no_argument, 0, 'L'},
       {"verbose", no_argument, 0, 'v'},
       {"version", no_argument, 0, 'V'},
       {0, 0, 0, 0}
       };
 
-    int c = getopt_long( argc, argv, "14ab:c:CD:fF:hiLl:o:R:S:T:Vvx:",
+    int c = getopt_long( argc, argv, "14ab:c:CD:fF:hil:o:R:S:T:Vvx:",
                          long_options, 0 );
     if( c == -1 ) break;		// all options processed
 
@@ -257,7 +275,6 @@ int main( int argc, char *argv[] ) throw()
       case 'h': show_help( argv[0], control.verbose ); return 0;
       case 'i': control.invert = true; break;
       case 'l': control.layout_level = std::strtol( optarg, 0, 0 ); break;
-      case 'L': control.level = true; break;
       case 'o':	outfile_name = optarg; break;
       case 'R': control.scale = std::strtol( optarg, 0, 0 ); break;
       case 'S': control.specialtype = optarg[0]; break;
@@ -269,7 +286,7 @@ int main( int argc, char *argv[] ) throw()
       }
     } // end process options
 
-  if( outfile_name )
+  if( outfile_name && std::strcmp( outfile_name, "-" ) != 0 )
     {
     if( control.append ) control.outfile = std::fopen( outfile_name, "a" );
     else if( control.force ) control.outfile = std::fopen( outfile_name, "w" );
@@ -282,11 +299,20 @@ int main( int argc, char *argv[] ) throw()
       { std::fprintf( stderr, "Cannot open %s\n", outfile_name ); return 1; }
     }
 
-  if( exportfile_name )
+  if( exportfile_name && control.debug_level == 0 && control.scale == 0 &&
+      control.specialtype == 0 && !control.copy )
     {
-    control.exportfile = std::fopen( exportfile_name, "w" );
-    if( !control.exportfile )
-      { std::fprintf( stderr, "Cannot open %s\n", exportfile_name ); return 1; }
+    if( std::strcmp( exportfile_name, "-" ) == 0 )
+      { control.exportfile = stdout; if( !outfile_name ) control.outfile = 0; }
+    else
+      {
+      control.exportfile = std::fopen( exportfile_name, "w" );
+      if( !control.exportfile )
+        {
+        std::fprintf( stderr, "Cannot open %s\n", exportfile_name );
+        return 1;
+        }
+      }
     std::fprintf( control.exportfile,
                   "# Ocr Results File. Created by GNU Ocrad version %s\n",
                   program_version );
@@ -323,10 +349,10 @@ int main( int argc, char *argv[] ) throw()
     int tmp = process_file( infile, charset, control );
     if( tmp > 0 && infile == stdin ) infile = 0;
     if( tmp > retval ) retval = tmp;
-    std::fflush( control.outfile );
+    if( control.outfile ) std::fflush( control.outfile );
     if( control.exportfile ) std::fflush( control.exportfile );
     }
-  std::fclose( control.outfile );
+  if( control.outfile ) std::fclose( control.outfile );
   if( control.exportfile ) std::fclose( control.exportfile );
   return retval;
   }
