@@ -19,7 +19,7 @@
 class Profile
   {
 public:
-  enum type { left, top, right, bottom, height, width, zheight, zwidth };
+  enum type { left, top, right, bottom, height, width };
 
 private:
   const Block * _block;		// Block to witch this profile belongs
@@ -48,8 +48,7 @@ public:
   int operator[]( int i ) throw();
   int pos( int p ) throw();
   int range() throw();
-  int samples() throw()
-    { if( !valid ) initialize(); return data.size(); }
+  int samples() throw() { if( !valid ) initialize(); return data.size(); }
 
   bool increasing( int i = 1 ) throw();
   bool decreasing( int i = 1 ) throw();
