@@ -91,14 +91,6 @@ void Rectangle::bottom( int b ) throw()
   }
 
 
-int Rectangle::hpos( int p ) const throw()
-  { return ( ( ( _right - _left ) * p ) / 100 ) + _left; }
-
-
-int Rectangle::vpos( int p ) const throw()
-  { return ( ( ( _bottom - _top ) * p ) / 100 ) + _top; }
-
-
 void Rectangle::add_point( int row, int col ) throw()
   {
   if( row > _bottom ) _bottom = row; else if( row < _top ) _top = row;
@@ -167,6 +159,19 @@ bool Rectangle::v_overlaps( const Rectangle & r ) const throw()
   { return ( _top <= r._bottom && _bottom >= r._top ); }
 
 
+bool Rectangle::h_precedes( const Rectangle & r ) const throw()
+  { return ( hcenter() < r.hcenter() ); }
+
+
+bool Rectangle::v_precedes( const Rectangle & r ) const throw()
+  {
+  if( _bottom < r.vcenter() || vcenter() < r._top ) return true;
+  if( this->includes_vcenter( r ) && r.includes_vcenter( *this ) )
+    return this->h_precedes( r );
+  return false;
+  }
+
+
 int Rectangle::distance( const Rectangle & r ) const throw()
   { return hypoti( h_distance( r ), v_distance( r ) ); }
 
