@@ -21,13 +21,15 @@ class Textblock
   mutable std::vector< Textline > data;
 
   void add_line() throw() { data.push_back( Textline() ); }
-  void insert_line( int ) throw();
-  void delete_line( int ) throw();
+  void insert_line( int i ) throw();
+  void delete_line( int i ) throw();
+  void join_characters() throw();
 
 public:
-  Textblock( const Charset & charset, const std::vector< Block > & block_vector ) throw();
+  Textblock( const Charset & charset,
+             const std::vector< std::vector< Block > > & block_matrix ) throw();
 
-  Textline & line( int ) const throw();
+  Textline & line( int i ) const throw();
   int lines() const throw() { return data.size(); }
   int characters() const throw();
 
