@@ -21,6 +21,8 @@
 #include <vector>
 #include "common.h"
 #include "rectangle.h"
+#include "vrhomboid.h"
+#include "track.h"
 #include "ucs.h"
 #include "block.h"
 #include "character.h"
@@ -39,17 +41,44 @@ int Textline::find_space( int begin ) const throw()
   }
 
 
-const Character & Textline::add_character( const Character & c ) throw()
+Textline::Textline( const Textline & l ) throw()
+  : data( l.data ), track( l.track )
   {
-  data.push_back( c ); return data.back();
+  if( l._big_initial ) _big_initial = new Character( *(l._big_initial) );
+  else _big_initial = 0;
   }
 
 
-void Textline::insert_character( int i, const Character & c ) throw()
+Textline & Textline::operator=( const Textline & l ) throw()
   {
-  if( i < 0 || i > characters() )
-    Ocrad::internal_error( "insert_character, index out of bounds" );
-  data.insert( data.begin() + i, c );
+  if( this != &l )
+    {
+    delete_big_initial();
+    if( l._big_initial ) _big_initial = new Character( *(l._big_initial) );
+    data = l.data; track = l.track;
+    }
+  return *this;
+  }
+
+
+void Textline::initialize_track() throw()
+  {
+  std::vector< Rectangle > rv;
+  for( unsigned int i = 0; i < data.size(); ++i ) rv.push_back( data[i] );
+  track.initialize( rv );
+  }
+
+
+void Textline::big_initial( const Character & c ) throw()
+  {
+  if( !_big_initial ) _big_initial = new Character( c );
+  else *_big_initial = c;
+  }
+
+
+void Textline::delete_big_initial() throw()
+  {
+  if( _big_initial ) { delete _big_initial; _big_initial = 0; }
   }
 
 
@@ -61,22 +90,31 @@ void Textline::delete_character( int i ) throw()
   }
 
 
-const Character & Textline::shift_character( const Character & c ) throw()
+int Textline::shift_character( const Character & c ) throw()
   {
-  int i;
+  int i = characters();
 
-  for( i = characters(); i > 0; --i )
-    if( c.left() >= data[i-1].left() ) break;
+  while( i > 0 && c.h_precedes( data[i-1] ) ) --i;
   data.insert( data.begin() + i, c );
-  return data[i];
+  return i;
   }
 
 
-void Textline::swap_characters( int i, int j ) throw()
+bool Textline::insert_space( int i ) throw()
   {
-  if( i < 0 || i >= characters() || j < 0 || j >= characters() )
-    Ocrad::internal_error( "swap_characters, index out of bounds" );
-  Character tmp = data[i]; data[i] = data[j]; data[j] = tmp;
+  if( i <= 0 || i >= characters() )
+    Ocrad::internal_error( "insert_space, index out of bounds" );
+  Character & c1 = data[i-1];
+  Character & c2 = data[i];
+  if( c2.left() - c1.right() <= 1 ) return false;
+  int left = c1.right() + 1;
+  int top = ( c1.top() + c2.top() ) / 2;
+  int right = c2.left() - 1;
+  int bottom = ( c1.bottom() + c2.bottom() ) / 2;
+  Character c( Rectangle( left, top, right, bottom ), *c1.blockmap() );
+  c.add_guess( ' ', 0 );
+  data.insert( data.begin() + i, c );
+  return true;
   }
 
 
@@ -92,19 +130,9 @@ int Textline::mean_height() const throw()
   {
   int sum = 0;
 
-  if( characters() == 0 ) return sum;
   for( int i = 0; i < characters(); ++i ) sum += data[i].height();
-  return sum / characters();
-  }
-
-
-int Textline::mean_height1() const throw()
-  {
-  int sum = 0;
-
-  if( characters() <= 1 ) return sum;
-  for( int i = 1; i < characters(); ++i ) sum += data[i].height();
-  return sum / ( characters() - 1 );
+  if( characters() ) sum /= characters();
+  return sum;
   }
 
 
@@ -112,9 +140,9 @@ int Textline::mean_width() const throw()
   {
   int sum = 0;
 
-  if( characters() == 0 ) return sum;
   for( int i = 0; i < characters(); ++i ) sum += data[i].width();
-  return sum / characters();
+  if( characters() ) sum /= characters();
+  return sum;
   }
 
 
@@ -134,9 +162,9 @@ int Textline::mean_hcenter() const throw()
   {
   int sum = 0;
 
-  if( characters() == 0 ) return sum;
   for( int i = 0; i < characters(); ++i ) sum += data[i].hcenter();
-  return sum / characters();
+  if( characters() ) sum /= characters();
+  return sum;
   }
 
 
@@ -144,22 +172,27 @@ int Textline::mean_vcenter() const throw()
   {
   int sum = 0;
 
-  if( characters() == 0 ) return sum;
   for( int i = 0; i < characters(); ++i ) sum += data[i].vcenter();
-  return sum / characters();
+  if( characters() ) sum /= characters();
+  return sum;
   }
 
 
 void Textline::join( Textline & l ) throw()
   {
-  for( int i = 0; i < l.characters(); ++i )
-    shift_character( l.data[i] );
-  l.data.clear();
+  if( l._big_initial &&
+      ( !_big_initial || l._big_initial->h_precedes( *_big_initial ) ) )
+    big_initial( *(l._big_initial) );
+
+  for( int i = 0; i < l.characters(); ++i ) shift_character( l.data[i] );
+
+  l.delete_big_initial(); l.data.clear();
   }
 
 
 void Textline::print( const Control & control ) const throw()
   {
+  if( _big_initial ) _big_initial->print( control );
   for( int i = 0; i < characters(); ++i ) character( i ).print( control );
   std::fputs( "\n", control.outfile );
   }
@@ -169,16 +202,23 @@ void Textline::dprint( const Control & control, bool graph, bool recursive )
 								const throw()
   {
   if( graph || recursive )
-    std::fprintf( control.outfile, "mean_height = %d\n", mean_height() );
+    std::fprintf( control.outfile, "mean_height = %d, track.segments = %d\n",
+                  mean_height(), track.segments() );
 
+  if( _big_initial ) _big_initial->dprint( control, graph, recursive );
   for( int i = 0; i < characters(); ++i )
+    {
+    if( graph || recursive )
+      std::fprintf( control.outfile, "track.vcenter = %d\n", track.vcenter( character(i).hcenter() ) );
     character( i ).dprint( control, graph, recursive );
+    }
   std::fputs( "\n", control.outfile );
   }
 
 
 void Textline::xprint( const Control & control ) const throw()
   {
+  if( _big_initial ) _big_initial->xprint( control );
   for( int i = 0; i < characters(); ++i )
     character( i ).xprint( control );
   }
@@ -186,31 +226,21 @@ void Textline::xprint( const Control & control ) const throw()
 
 void Textline::recognize1( const Charset & charset ) const throw()
   {
-  if( characters() == 0 ) return;
-  int mh = mean_height(), l, r;
-
-  for( l = 0; l < characters() / 2; ++l )
-    if( Ocrad::similar( character( l ).height(), mh, 20 ) ) break;
-  for( r = characters() - 1; r > characters() / 2; --r )
-    if( Ocrad::similar( character( r ).height(), mh, 20 ) ) break;
-  if( r - l < characters() / 2 )
+  if( _big_initial )
     {
-    for( l = 0; l < characters() / 2; ++l )
-      if( character( l ).height() > mh ) break;
-    for( r = characters() - 1; r > characters() / 2; --r )
-      if( character( r ).height() > mh ) break;
-    if( r - l < characters() / 2 ) { l = 0; r = characters() - 1; }
+    _big_initial->recognize1( charset, _big_initial->vcenter() );
+    if( _big_initial->guesses() == 1 )
+      {
+      int code = _big_initial->guess( 0 ).code;
+      if( UCS::islower( code ) )
+        _big_initial->only_guess( UCS::toupper( code ), 0 );
+      }
+    else if( _big_initial->guesses() > 1 ) _big_initial->clear_guesses();
     }
 
-  int xl = character( l ).hcenter(), yl = character( l ).vcenter();
-  int xr = character( r ).hcenter(), yr = character( r ).vcenter();
-  int dx = xr - xl, dy = yr - yl;
   for( int i = 0; i < characters(); ++i )
     {
     Character & c = character( i );
-    int charbox_vcenter = yl;
-    if( dx )
-      charbox_vcenter += ( dy * ( c.hcenter() - xl ) / dx );
-    c.recognize1( charset, charbox_vcenter );
+    c.recognize1( charset, track.vcenter( c.hcenter() ) );
     }
   }
