@@ -18,30 +18,40 @@
 
 class Textline
   {
+  Character * _big_initial;
   mutable std::vector< Character > data;
+  Track track;
   int find_space( int begin ) const throw();
 
 public:
-  Textline() throw() {}
+  Textline() throw() : _big_initial( 0 ) {}
+  Textline( const Textline & l ) throw();
+  Textline & operator=( const Textline & l ) throw();
+  ~Textline() throw() { delete_big_initial(); }
+  void initialize_track() throw();
+
+  const Character * big_initial() const throw() { return _big_initial; }
+  void big_initial( const Character & c ) throw();
+  void delete_big_initial() throw();
 
-  const Character & add_character( const Character & c ) throw();
-  void insert_character( int i, const Character & ) throw();
   void delete_character( int i ) throw();
-  const Character & shift_character( const Character & c ) throw();
-  void swap_characters( int i, int j ) throw();
+  int  shift_character( const Character & c ) throw();
+  bool insert_space( int i ) throw();
 
   Character & character( int i ) const throw();
-  const Character & front() const throw() { return data.front(); }
-  const Character & back() const throw() { return data.back(); }
   int characters() const throw() { return data.size(); }
 
   int mean_height() const throw();
-  int mean_height1() const throw();
   int mean_width() const throw();
   int mean_gap_width() const throw();
   int mean_hcenter() const throw();
   int mean_vcenter() const throw();
 
+  int height() const throw() { return track.height(); }
+  int vcenter( int col ) const throw() { return track.vcenter( col ); }
+  int bottom( int col ) const throw() { return track.bottom( col ); }
+  int top( int col ) const throw() { return track.top( col ); }
+
   void join( Textline & l ) throw();
   void print( const Control & control ) const throw();
   void dprint( const Control & control, bool graph, bool recursive ) const throw();
