@@ -21,6 +21,8 @@
 #include <vector>
 #include "common.h"
 #include "rectangle.h"
+#include "vrhomboid.h"
+#include "track.h"
 #include "ucs.h"
 #include "block.h"
 #include "blockmap.h"
@@ -47,11 +49,11 @@ void Textline::recognize2( const Charset & charset ) throw()
       if( ( c1.guesses() && c2.guesses() ) ||
           Ocrad::similar( c1.height(), c2.height(), 20 ) )
         {
-        if( c1.left() <= c2.left() ) c = c1;
+        if( c1.height() > c2.height() ) c = c1;
         else { c = c2; c2 = c1; }
         // discards spurious dots
         if( !c2.maybe('.') || c2.top() > c.vcenter() )
-          { insert_character( i + 1, c2 ); ++i; }
+          { shift_character( c2 ); ++i; }
         }
       }
     }
@@ -60,14 +62,19 @@ void Textline::recognize2( const Charset & charset ) throw()
   for( int i = 0; i < characters(); ++i )
     {
     Character & c = character( i );
-    if( !c.guesses() && c.blocks() == 1 && c.width() > 20 &&
+    if( !c.guesses() && c.width() > 20 &&
         Ocrad::similar( c.width(), 2 * mean_width(), 25 ) &&
-        Ocrad::similar( c.height(), mean_height1(), 30 ) )
+        Ocrad::similar( c.height(), mean_height(), 30 ) )
       {
-      const Block & b = c.block( 0 );
+      int ib = c.blocks() - 1;
+      for( int k = ib - 1; k >= 0; --k )
+        if( c.block( k ).width() > c.block( ib ).width() ) ib = k;
+      if( ib < 0 || 10 * c.block( ib ).width() < 9 * c.width() )
+        continue;
+      const Block & b = c.block( ib );
       const Blockmap & bm = *b.blockmap();
       int cmin = 0, hmin = b.height() + 1;
-      for( int col = b.hpos(40); col <= b.hpos(60); ++col)
+      for( int col = b.hpos(33); col <= b.hpos(66); ++col)
         {
         int h = 0;
         for( int row = b.top(); row <= b.bottom(); ++row )
@@ -82,40 +89,45 @@ void Textline::recognize2( const Charset & charset ) throw()
       Block b2( r2, bm, b.id() ); b2.adjust_height();
       for( int j = 0; j < b.blocks(); ++j )
         {
-        if( b1.includes( b.block( j ) ) ) b1.shift_block( b.block( j ) );
-        else if( b2.includes( b.block( j ) ) ) b2.shift_block( b.block( j ) );
+        const Block & bj = b.block( j );
+        if( b1.includes( bj ) ) b1.shift_block( bj );
+        else if( b2.includes( bj ) ) b2.shift_block( bj );
         }
-      Character c1( b1 ); c1.recognize1( charset, c.vcenter() );
-      Character c2( b2 ); c2.recognize1( charset, c.vcenter() );
+      Character c1( b1 );
+      Character c2( b2 );
+      for( int j = 0; j < c.blocks(); ++j ) if( j != ib )
+        {
+        const Block & bj = c.block( j );
+        if( c1.includes_hcenter( bj ) ) c1.shift_block( bj );
+        else if( c2.includes_hcenter( bj ) ) c2.shift_block( bj );
+        }
+      c1.recognize1( charset, c.vcenter() );
+      c2.recognize1( charset, c.vcenter() );
       if( c1.guesses() || c2.guesses() )
-        { c = c1; insert_character( i + 1, c2 ); ++i; }
+        { c = c1; shift_character( c2 ); ++i; }
       }
     }
 
   // separate merged characters recognized by recognize1
-  for( int i = 0; i < characters(); ++i )	// FIXME this block sucks
+  for( int i = 0; i < characters(); )		// FIXME this block sucks
     {
-    int j = i, guesses = data[j].guesses();
-    if( guesses >= 1 && data[j].guess( 0 ).code == 0 )
+    if( !data[i].guesses() || data[i].guess( 0 ).code != 0 ) { ++i; continue; }
+    Character c( data[i] );
+    delete_character( i );
+    if( c.guesses() >= 3 && c.blocks() >= 1 )
       {
-      if( guesses >= 3 && data[j].blocks() >= 1 )
+      int left = c.guess( 0 ).value;
+      for( int g = 1; g < c.guesses(); ++g )
         {
-        int left = data[j].guess( 0 ).value;
-        for( int g = 1; g < guesses; ++g )
-          {
-          const Block & b = data[j].block( data[j].blocks() - 1 );
-          Rectangle r( left, b.top(), data[j].guess( g ).value, b.bottom() );
-          Character c1( Block( r, *data[j].blockmap(), b.id() ) );
-          for( int k = 0; k < data[j].blocks(); ++k )
-            if( !data[j].block( k ).includes( r ) && r.includes_hcenter( data[j].block( k ) ) )
-              c1.shift_block( data[j].block( k ) );
-          c1.add_guess( data[j].guess( g ).code, 0 );
-          left = r.right() + 1;
-          insert_character( j + g, c1 );
-          }
-        i += guesses - 2;
+        const Block & b = c.block( c.blocks() - 1 );
+        Rectangle r( left, b.top(), c.guess( g ).value, b.bottom() );
+        Character c1( Block( r, *(c.blockmap()), b.id() ) );
+        for( int k = 0; k < c.blocks(); ++k )
+          if( !c.block( k ).includes( r ) && r.includes_hcenter( c.block( k ) ) )
+            c1.shift_block( c.block( k ) );
+        c1.add_guess( c.guess( g ).code, 0 ); shift_character( c1 );
+        left = r.right() + 1;
         }
-      delete_character( j );
       }
     }
 
@@ -237,12 +249,6 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // transform a small 'p' to a capital 'P'
-  if( characters() > 2 )
-    {
-    Character & c = character( 0 );
-    if( c.guesses() == 1 && c.guess( 0 ).code == 'p' &&
-        c.height() > 2 * mean_height() ) c.only_guess( 'P', 0 );
-    }
   for( int i = characters() - 1; i > 0; --i )
     {
     Character & c1 = character( i - 1 );
